using System;
using System.Collections.Generic;
using System.Text;

namespace CricketScores
{
    /// <summary>
    /// Shows value type behaviour of structures
    /// </summary>
    class ScoreCalculator
    {
        #region Player structure

        /// <summary>
        /// Holds the details of a player in a game of cricket
        /// </summary>
        struct Player
        {
            #region Data Fields

            public string PlayerName;
            public int ScoreInRuns;
            public int InningsLengthInMins;

            #endregion

            #region Value type equality behaviour

            public static bool operator == (Player a, Player b)
            {
                // If both are null, or both are same instance, return true.
                if (System.Object.ReferenceEquals(a, b))
                {
                    return true;
                }

                // If one is null, but not both, return false.
                if (((object)a == null) || ((object)b == null))
                {
                    return false;
                }

                // Return true if the fields match:
                return a.PlayerName == b.PlayerName && 
                    a.ScoreInRuns == b.ScoreInRuns && 
                    a.InningsLengthInMins == b.InningsLengthInMins;
            }

            public static bool operator !=(Player a, Player b)
            {
                // If both are null, or both are same instance, return true.
                if (System.Object.ReferenceEquals(a, b))
                {
                    return false;
                }

                // If one is null, but not both, return false.
                if (((object)a == null) || ((object)b == null))
                {
                    return true;
                }

                // Return true if the fields match:
                return a.PlayerName != b.PlayerName ||
                    a.ScoreInRuns != b.ScoreInRuns ||
                    a.InningsLengthInMins != b.InningsLengthInMins;
            }

            #endregion


            #region Constructors

            public Player(string inPlayerName, int inScoreInRuns, int inInningsLengthInMins)
            {
                PlayerName = inPlayerName;
                ScoreInRuns = inScoreInRuns;
                InningsLengthInMins = inInningsLengthInMins;
            }

            #endregion

            #region Overridden Methods

            public override string ToString()
            {
                return PlayerName + " Score: " + ScoreInRuns + 
                    " Length: " + InningsLengthInMins +
                    " mins";
            }

            public override bool Equals(object obj)
            {
                return base.Equals(obj);
            }

            public override int GetHashCode()
            {
                return base.GetHashCode() +
                    PlayerName.GetHashCode() +
                    ScoreInRuns.GetHashCode() +
                    InningsLengthInMins.GetHashCode();
            }

            #endregion
        }

        #endregion


        private static Player[] sampleTeam = new Player[] {
            new Player ( "Rob Miles", 200, 90 ),
            new Player ( "W C Grace", 100, 90 ),
            new Player ( "Ian Botham", 0, 10 ),
            new Player ( "Freddie Flintoff", 20, 350 ),
            new Player ( "David Beckham", 0, 45 ),
            new Player ( "Victoria Beckam", 75, 120 ),
            new Player ( "Simon Cowell", 5, 10 ),
            new Player ( "Red Rum", 76, 60 ),
            new Player ( "Band of the Argyle Scottish Highlanders", 23, 67 ),
            new Player ( "The Blob", 12, 1 ),
            new Player ( "David Grey", 150, 120 )
        };

        private static void bubbleSortOnScore(Player[] array)
        {
            for (int passNo = 0; passNo < array.Length; passNo++)
            {
                for (int player = 0; player < (array.Length-1); player++)
                {
                    if (array[player].ScoreInRuns < array[player + 1].ScoreInRuns)
                    {
                        Player temp = array[player];
                        array[player] = array[player + 1];
                        array[player + 1] = temp;
                    }
                }
            }
        }

        static void Main(string[] args)
        {
            Console.WriteLine("Before Sort:");
            foreach ( Player player in sampleTeam ) {
                Console.WriteLine(player);
            }

            bubbleSortOnScore(sampleTeam);

            Console.WriteLine("After Sort:");
            foreach (Player player in sampleTeam)
            {
                Console.WriteLine(player);
            }
        }
    }
}
