using System;
using System.Collections.Generic;
using System.Text;

namespace GenericDemo
{
    /// <summary>
    /// This a node in our linked list
    /// </summary>
    /// <typeparam name="T">type of the data payload in the list</typeparam>
    public class Node<T>
    {
        // data payload of the node in the list
        // of type "undetermined when the list is being created"
        T data;

        // link to the next node in the list
        // must be a reference of type ListNode for whatever type
        // the ListNode is built on
        Node<T> next;

        /// <summary>
        /// Implement the data as a property of the node
        /// </summary>
        public T Data
        {
            get
            {
                return this.data;
            }
            set
            {
                this.data = value;
            }
        }

        /// <summary>
        /// Implenet the next reference as a property
        /// </summary>
        public Node<T> Next
        {
            get
            {
                return this.next;
            }
            set
            {
                this.next = value;
            }
        }

        /// <summary>
        /// Creates an instance of a node which is not part of a list
        /// </summary>
        /// <param name="inData">The data payload for this node</param>
        public Node(T inData)
        {
            Data = inData;
            Next = null;
        }

        /// <summary>
        /// Adds a node on to the end of this one. Spins down to 
        /// the end of the list.
        /// </summary>
        /// <param name="newNode">node to add</param>
        public void Append ( Node<T> newNode ) 
        {
            if (this.next == null)
            {
                this.next = newNode;
            }
            else
            {
                this.next.Append(newNode);
            }
        }

        public override string ToString()
        {
            return data.ToString();
        }
    }

    /// <summary>
    /// A linked list of types
    /// </summary>
    /// <typeparam name="T">the type of the data payload of each list element</typeparam>
    public class LinkedList<T>
    {
        Node<T> listHead = null;

        public void Add(T inData)
        {
            if (listHead == null)
            {
                listHead = new Node<T>(inData);
            }
            else
            {
                listHead.Append(new Node<T>(inData));
            }
        }

        public bool Delete(T inData)
        {
            // If the list is empty we have failed
            if (listHead == null)
            {
                return false;
            }

            // if the first node is a match, easy delete
            if (listHead.Data.Equals(inData))
            {
                listHead = listHead.Next;
                return true;
            }

            // need to search the list for a data match
            Node<T> currentNode = listHead;

            do {
                Node<T> nextNode = currentNode.Next ;
                if ( nextNode == null ) {
                    return false;
                }
                if ( nextNode.Data.Equals(inData) ) {
                    currentNode.Next = nextNode.Next;
                    return true;
                }
                currentNode = nextNode;
            } while ( true ) ;
        }

        public override string ToString()
        {
            string result = "";
            Node<T> node = listHead;
            while (node != null)
            {
                result += node.Data.ToString() + "\n";
                node = node.Next;
            }
            return result;
        }
    }


    class Program
    {
        static void Main(string[] args)
        {
            LinkedList<int> intLinkedList = new LinkedList<int>();

            intLinkedList.Add(1);
            intLinkedList.Add(2);
            intLinkedList.Add(3);
            intLinkedList.Add(4);
            intLinkedList.Add(5);
            Console.WriteLine (intLinkedList);

            intLinkedList.Delete(3);
            Console.WriteLine(intLinkedList);

            LinkedList<string> stringLinkedList = new LinkedList<string>();

            stringLinkedList.Add("one");
            stringLinkedList.Add("two");
            stringLinkedList.Add("three");
            stringLinkedList.Add("four");
            stringLinkedList.Add("five");
            Console.WriteLine(stringLinkedList);

            stringLinkedList.Delete("one");
            Console.WriteLine(stringLinkedList);

            stringLinkedList.Delete("five");
            Console.WriteLine(stringLinkedList);

            Console.ReadLine();

        }
    }


}
